<?php
/**
 * "Archiv" Plugin zum Ausgeben eines Jahres-/Monatsarchivs in der Seitenleiste
 *
 * @package     Expblog
 * @category    Plugins
 * @author      Volker S. Latainski
 * @copyright   Copyright (c) 2008, Volker S. Latainski
 * @version     0.6.2
 * @license     Freeware (siehe LIZENZ.txt im Hauptverzeichnis)
 * @link        http://www.expblog.net
 * @link        http://www.expblog.de
 * @filesource
 */

/**
 * Hauptfunktion des Plugins
 *
 * @param   object  $output  Referenz(keine Kopie!) auf das Output Objekt
 * @param   array   $lang    Array der aus der aktuell verwendeten Sprachdatei
 * @param   array   $post    Kopie von $_POST
 * @param   array   $get     Kopie von $_GET
 * @return  void
 */
function archiv($output, $lang, $post, $get)
{
  if($output->config['dbms'] == 'mysql')
  {
    $term1 = "YEAR(FROM_UNIXTIME(news.timestamp))";
    $term2 = "MONTH(FROM_UNIXTIME(news.timestamp))";
  }
  elseif($output->config['dbms'] == 'sqlite')
  {
    $term1 = "strftime('%Y', news.timestamp, 'unixepoch')";
    $term2 = "strftime('%m', news.timestamp, 'unixepoch')";
  }
  else
  {
    return 0;
  }
  $res = $output->sql->query("
    SELECT
      {$term1} as jahr,
      {$term2} as monat,
      COUNT(news.id) as count
    FROM
      ".DB_TABLE_news." AS news
    WHERE
      news.published = 1
    GROUP BY
      jahr, monat
    ORDER BY
      jahr DESC, 
      monat DESC   
  ");

  $archiv = array();
  foreach($res as $array => $row)
  {
    $archiv[$row['jahr']][$row['monat']]['count'] = $row['count'];
  }
  unset($res);
  if(count($archiv) > 0):
?>
  <!-- Anfang Archiv Plugin -->
  <div id="plugin_archiv">
    <p id="plugin_archiv_topic"><?=$lang['plugins']['archiv']['title']?></p>
    <dl id="plugin_archiv_liste">
    <?php
      foreach($archiv as $jahr => $monat)
      {
        echo '<dt>'.$jahr.'</dt>';
        foreach($monat as $monat => $count)
        {
          $mts = mktime(0, 0, 0, ($monat+1), 0, $jahr);
          if($output->config['modrewrite'])
          {
            echo '<dd><a href="'.WEBPATH.'archiv/'.$jahr.'/'.$monat.'/">'.strftime("%B", $mts).'</a> ('.$count['count'].')</dd>';
          }
          else
          {
            echo '<dd><a href="'.WEBPATH.'index.php?a=archiv&amp;j='.$jahr.'&amp;m='.$monat.'">'.strftime("%B", $mts).'</a> ('.$count['count'].')</dd>';
          }
        }
      }
      unset($archiv);
    ?>
    </dl>
  </div>
  <!-- Ende Archiv Plugin -->
<?php endif; } ?>