<?php
/**
 * Captcha Klasse
 *
 * Diese Klasse stellt den Captcha fr das Expblog zur Verfgung.
 * @package     Expblog
 * @category    Backend
 * @author      Volker S. Latainski (based on TMY's Captcha. Danke btw!)
 * @copyright   Copyright (c) 2008, Volker S. Latainski
 * @version     0.6
 * @license     Freeware (siehe LIZENZ.txt im Hauptverzeichnis)
 * @link        http://www.expblog.net
 * @link        http://www.expblog.de
 * @filesource
 */
class CAPTCHA
{
  /**
   * Pfad zum Ordner fr temporre Dateien. Ausgehend vom system Ordner.
   * @var string
   */
  private $tempPath = 'tmp/';
  /**
   * Dateiname der Captcha TTF-Schrift. Ausgehend vom system Ordner.
   * @var string
   */
  private $font = 'fonts/captcha.ttf';
  /**
   * Schriftgrssedes Captchas.
   * @var int
   */
  private $fontsize = 18;
  /**
   * Pfad zum Fallback Captcha Bild. Ausgehend vom system Ordner.
   * @var string
   */
  private $bgImage = 'images/captchabg.png';
  /**
   * Fallback Schriftzug des Captchas.
   * @var string
   */
  private $fakeText = 'expblog';
  /**
   * Lnge des Capcha Hashs.
   * @var int
   */
  private $hashLenght = 32;
  /**
   * Lnge der User Eingabe bzw. des Captchas.
   * @var int
   */
  private $captchaLenght = 5;
  /**
   * Gltigkeitsdauer des Captcha in Sekunden.
   * @var int
   */
  private $lifetime = 600;
  /**
   * Verfgbare Zeichen im Captcha.
   * @var array
   */
  private $captchaChars = array(
    "1", "2", "3", "4", "5", "6", "7", "8", "9",
    "A", "B", "C", "D", "E", "F", "G", "H", "J",
    "K", "L", "M", "N", "P", "R", "S", "T", "U",
    "V", "W", "X", "Y", "Z"
  );

  /**
   * Konstruktor
   *
   * @return   void
   * @access   public
   */
  public function __construct() {}

  /**
   * Funktion zum Erzeugen eines neuen Captchas.
   *
   * @param    string $captchaPath Pfad zum Temp-Verzeichnis der Captchas. Fallback=system/tmp
   * @return   int
   * @access   public
   */
  public function newCaptcha($captchaPath='')
  {
    if(empty($captchaPath)) 
    {
      $captchaPath = $this->tempPath;
    }
    
    $captcha_session_code = substr(md5(uniqid(microtime())), 0, $this->hashLenght);
    $captchaFile = $captchaPath.$captcha_session_code.'.txt';
    $captchaText = '';
    
    for($i=0;$i<=($this->captchaLenght-1);$i++)
    {
      $randomChar = $this->captchaChars[rand(0,count($this->captchaChars)-1)];
      $captchaText .= $randomChar;
    }

    if(!$handle = @ fopen($captchaFile, "w"))
    {
      return 0;
    }
    
    if(!@ fwrite($handle, $captchaText))
    {
      return 0;
    }
    
    @ fclose($handle);

    return $captcha_session_code;
  }

  /**
   * Funktion zum Erzeugen eines Default/Fake Captchas. Wenn irgenmdwas falsch luft wird dieser 
   * Captcha angezeigt.
   *
   * @return   void
   * @access   public
   */
  public function fakeCaptcha()
  {
    $img = ImageCreateFromPNG($this->bgImage);
    $color = ImageColorAllocate($img, 123, 217, 255);  // font color
    $angle = 0;  // rotation
    $t_x = 8;  // position
    $t_y = 28;  // position
    header('Content-type: image/png');
    imagettftext($img, $this->fontsize, $angle, $t_x, $t_y, $color, $this->font, $this->fakeText);
    imagepng($img);
    imagedestroy($img);
  }

  /**
   * Ausgabe des Captchas. Diese Funktion gibt das Captchabild aus. 
   *
   * @param    string $captchaHash Hashwert des Captchas. Dient der Identifizierung des Captchas.
   * @return   void|image
   * @access   public
   */
  public function showCaptcha($captchaHash)
  {
    $captchaFile = $this->tempPath.$captchaHash.".txt";
    if (is_readable($captchaFile))
    {
      $img = ImageCreateFromPNG($this->bgImage);
      $color = ImageColorAllocate($img, 123, 217, 255);  // font color
      $angle = rand(0,5);  // rotation
      $t_x = rand(10,20);  // position
      $t_y = rand(28,33);  // position
      $text = file($captchaFile);

      header('Content-type: image/png');
      imagettftext($img, $this->fontsize, $angle, $t_x, $t_y, $color, $this->font, $text[0]);
      imagepng($img);
      imagedestroy($img);
    }
    else
    {
      $this->fakeCaptcha();
    }
  }

  /**
   * Validierung der Usereingabe zu einem Captcha. Diese Funktion berprft die Usereingabe und 
   * vergleicht sie mit dem Captcha.
   *
   * @param    string $captcha Usereingabe
   * @param    string $hash Hashwert des Captchas. Dient der Identifizierung des Captchas
   * @param    string $captchaPath Pfad zum Temp-Verzeichnis der Captchas. Fallback=system/tmp
   * @return   int
   * @access   public
   */
  public function checkCaptcha($captcha, $hash, $captchaPath = '')
  {
    $result = array(2,0,0);
    $captcha = strtoupper(trim($captcha));
    $hash = trim($hash);
    if(empty($captchaPath)) 
    {
      $captchaPath = $this->tempPath;
    }
    $filename = $captchaPath.$hash.".txt";
    if(strlen($hash) == $this->hashLenght)
    {
      if (@ file_exists($filename)) // file check
      {
        $captchaArray = file($filename);
        if (rtrim($captchaArray[0]) == $captcha)
        {          
          return 1;  // ok
        }
        else
        {
          return 2;  // captcha mismatch
        }
      }
      else
      {
        return 3;  // no such file
      }
    }
    else
    {
      return 4;  // invalid hash
    }
  }

  /**
   * Funktion zum Aufrumen des system/tmp Ordners.
   *
   * @param    string $captchaPath Pfad zum Temp-Verzeichnis der Captchas. Fallback=system/tmp
   * @return   int
   * @access   public
   */
  public function cleanTmp($captchaPath)
  {
    if ($handle = @ opendir($captchaPath))
    {
      while (false !== ($file = @ readdir($handle)))
      {
        if ($file != "." && $file != ".." && $file != "index.php")
        {
          if ((time() - filemtime($captchaPath.$file)) >= $this->lifetime)
          {
            @ unlink($captchaPath.$file);
          }
        }
      }
      @ closedir($handle);
    }
  }

  /**
   * Destruktor
   *
   * @return   void
   * @access   public
   */
  public function __destruct() {}
}
?>