<?php
/**
 * (Abstrakte) Datenbank Wrapper Klasse
 *
 * Diese Klasse ist ein Interface fuer die Datenbankanbindung.
 * @package     Expblog
 * @category    Backend
 * @author      Volker S. Latainski
 * @copyright   Copyright (c) 2008, Volker S. Latainski
 * @version     0.6
 * @license     Freeware (siehe LIZENZ.txt im Hauptverzeichnis)
 * @link        http://www.expblog.net
 * @link        http://www.expblog.de
 * @filesource
 */
abstract class DATABASE {

  /**
   * Kennung(Resource/Link) der Datenbankverbindung oder false.
   * @var bool|resource
   */
  private $db_link = false;
  /**
   * Anzahl der Queries.
   * @var int
   */
  private $db_queries = 0;


  /**
   * Baut die Verbindung zum Datenbank Server auf.
   *
   * @param    string $database Name der Datenbank
   * @param    string $host MYSQL Hostname (meist localhost)
   * @param    string $username MYSQL Benutzername (gibts vom Webhoster)
   * @param    string $password MYSQL Passwort (gibts auch vom Webhoster)
   * @return   bool
   * @access   public
   */
  abstract function connect($database, $host, $username, $password);


  /**
   * Funktion zum Selektieren der Datenbank.
   *
   * @param    string $database String mit dem Namen der zu benutzenden Datenbank
   * @return   bool
   * @access   public
   */
  abstract public function selectDB($database);


  /**
   * Funktion zur Datenbankabfrage.
   *
   * @param    string $statement String mit dem SQL Query
   * @param    bool $debug Optionaler Parameter fuer Debugging (Ausgabe des Querys)
   * @return   resource|bool
   * @access   public
   */
  abstract public function query($statement, $debug=false);
  
  
  /**
   * Funktion zum Validieren von Eingabe-Strings welche direkt in einem SQL-Query genutzt werden
   * Ungeprfte Eingaben sollten auf keinen Fall verwendet werden! (Schutz vor SQL Injection)
   *
   * @param    string $value String mit der Usereingabe, welche zu berprfen ist.
   * @return   string
   * @access   public
   */
  abstract public function checkSql($value);


  /**
   * Funktion zum Schliessen der Verbindung zum MYSQL Server.
   *
   * @param    resource $link Verbindungs-Kennung (Link) der MYSQL Verbindung zum Server
   * @return   bool
   * @access   public
   */
  abstract public function close($link);


  /**
   * Ausgabe der SQL Server Version.
   *
   * @param    resource $link Verbindungs-Kennung (Link) der Verbindung zum Server
   * @return   string
   * @access   public
   */
  abstract public function version($link);


  /**
   * Liefert die Anzahl aller bisherigen SQL Queries die gestellt wurden.
   *
   * @return   int Anzahl der SQL Queries
   * @access   public
   */
  public function getQueryCount()
  {
    return $this->db_queries;
  }


  /**
   * Liefert die Verbindungskennung der Datenbankverbindung.
   *
   * @return   resource Verbindungs-Kennung
   * @access   public
   */
  public function getDBLink()
  {
    return $this->db_link;
  }  

}
?>