<?php
/**
 * Basisklasse fr die Detailansicht
 *
 * Diese Klasse ist fr das Handling der Detailansicht zustndig.
 * @package     Expblog
 * @category    Backend
 * @author      Volker S. Latainski
 * @copyright   Copyright (c) 2008, Volker S. Latainski
 * @version     0.6.1
 * @license     Freeware (siehe LIZENZ.txt im Hauptverzeichnis)
 * @link        http://www.expblog.net
 * @link        http://www.expblog.de
 * @filesource
 */
class DETAILS extends INDEX
{
  /**
   * Hitcounter Objekt
   * @var object
   */
  public $hitcounter;
  /**
   * Captcha Objekt
   * @var object
   */
  public $captcha;
  /**
   * Kommentar Objekt
   * @var object
   */
  public $comments;
  /**
   * Statusflag ob der eingegebene Captcha gltig ist.
   * Wird nur bei aktiviertem Spamschutz verwendet.
   * @var bool
   */
  public $captcha_check;
  /**
   * Der bergebene Session Code zum Captcha
   * @var string
   */
  public $captcha_session_code;
  /**
   * Rohdaten als Array
   * @var array
   */
  public $news_raw_data;
  /**
   * ID des aufgerufenen Artikels
   * @var int
   */
  public $newsid, $news_id;
  /**
   * Startzeit/Timestamp des aufgerufenen Artikels
   * @var int
   */
  public $news_time;
  /**
   * Autor des aufgerufenen Artikels
   * @var string
   */
  public $news_author;
  /**
   * Autor des aufgerufenen Artikels mit gross geschriebenen Anfangsbuchstaben
   * @var string
   */
  public $news_author_ucf;
  /**
   * ID des Autor des aufgerufenen Artikels
   * @var int
   */
  public $news_userid;
  /**
   * Kategorie des aufgerufenen Artikels
   * @var int
   */
  public $news_kategorie;
  /**
   * Titel des aufgerufenen Artikels
   * @var string
   */
  public $news_titel;
  /**
   * Anreisser/Teaser Text des aufgerufenen Artikels
   * @var string
   */
  public $news_teaser;
  /**
   * Nachricht/Body des aufgerufenen Artikels
   * @var string
   */
  public $news_nachricht;
  /**
   * ID des vorherigen, gltigen Artikels
   * @var int
   */
  public $news_last_valid_id;
  /**
   * ID des nchsten, gltigen Artikels
   * @var int
   */
  public $news_next_valid_id;
  /**
   * Value fr den Usernamen im Kommentarformular
   * @var string
   */
  public $kommentar_name;
  /**
   * Value fr die Homepage im Kommentarformular
   * @var string
   */
  public $kommentar_homepage;

  /**
    * Konstruktor
    *
    * @param    array $expconf Kopie der Blogkonfiguration
    * @return   void
    * @access   public
    */
  public function __construct($expconf)
  {
    parent::__construct($expconf);
    $this->connectSQL();
    $this->checkCookies();
    $this->setNewsid();
    $this->setNewsDetails();
    $this->hitcounter = new HITCOUNTER($expconf);
    $this->comments = new COMMENTS($expconf);
    $this->captcha = new CAPTCHA();
    $this->createCaptcha();
    if($this->config['kommentare'] && !empty($_POST["kommentar"]) && !$this->checkLast($_POST["kommentar"],1))
    {
      $captcha_session_code = $this->sql->checkSql($_POST["captcha_session_code"]);
      $captcha_pwd = $this->sql->checkSql($_POST["captcha_pwd"]);

      if($this->config['spamschutz'])
      {
        if($this->captcha->checkCaptcha($captcha_pwd, $captcha_session_code, 'system/tmp/') === 1)
        {
          $this->comments->addComment($_POST["newsid"], 0, $_POST["kommentar"], $_POST["name"], time(), getenv('REMOTE_ADDR'), $_POST["homepage"]);
        }
      }
      else
      {
        $this->comments->addComment($_POST["newsid"], 0, $_POST["kommentar"], $_POST["name"], time(), getenv('REMOTE_ADDR'), $_POST["homepage"]);
      }
    }
  }

  /**
    * Artikel ID abholen und als Obejekt Eigenschaft setzen
    *
    * @return   void
    * @access   private
    */
  private function setNewsid()
  {
    //Beta mod_rewrite
    if($this->config['modrewrite'])
    {
      if(isset($_GET['a']) && $_GET['a'] === 'archiv' && isset($_GET['e']) && !empty($_GET['e']))
      {
        list($artikel_id, $title) = explode("-", htmlentities($_GET['e']), 2);
        $this->newsid = (int) $artikel_id;
      }
      else
      {
        $this->newsid = 0;
      }
    }
    else
    {
      if(isset($_GET['id']) && is_numeric($_GET['id']))
      {
        $this->newsid = (int) $_GET["id"];
      }
      else
      {
        $this->newsid = 0;
      }
    }
  }


  /**
    * Funktion zum Artikel auslesen und abspeichern der Daten im Details Objekt
    *
    * @return   bool
    * @access   public
    */
  public function setNewsDetails()
  {
    if($this->newsid > 0)
    {
      $res = $this->sql->query("
          SELECT
            news.*,
            user.name AS username,
            kategorien.name AS kategorie
          FROM
            ".DB_TABLE_news." AS news
          LEFT JOIN
            ".DB_TABLE_user." AS user
          ON
            user.id = news.userid
          LEFT JOIN
            ".DB_TABLE_kategorien." AS kategorien
          ON
            news.kategorie = kategorien.id
          WHERE
            news.id = {$this->newsid}
      ");
      if(is_array($res))
      {
        $this->news_raw_data = $res[0];
        $this->news_id = $res[0]['id'];
        $this->news_time = strftime('%A, %d. %B %Y', $res[0]['timestamp']);
        $this->news_year = date("Y", $res[0]['timestamp']);
        $this->news_month = date("m", $res[0]['timestamp']);
        $this->news_day = date("d", $res[0]['timestamp']);
        $this->news_author = $res[0]['username'];
        $this->news_author_ucf = ucfirst($res[0]['username']);
        $this->news_userid = $res[0]['userid'];
        $this->news_kategorie = $res[0]['kategorie'];
        $this->news_titel = $res[0]['titel'];
        $this->news_teaser = $this->videoParser(html_entity_decode($res[0]['teaser']));
        $this->news_nachricht = $this->videoParser(html_entity_decode($res[0]['nachricht']));
        $this->news_next_valid_id = $this->string2urlstring($this->getValidID('next'));
        $this->news_last_valid_id = $this->string2urlstring($this->getValidID('last'));
        return true;
      }
      return false;
    }
    else
    {
      return false;
    }
  }

  /**
    * Funktion fr das Erstellen und Validieren eines Captchas (Detailansicht des Blogs)
    *
    * @return   void
    * @access   public
    */
  public function createCaptcha()
  {
    if($this->config['spamschutz'])
    {
      if(isset($_POST["captcha_session_code"]))
      {
        $this->captcha_session_code = trim(htmlentities($_POST["captcha_session_code"]));
        $captcha_pwd = trim(htmlentities($_POST["captcha_pwd"]));
        if (file_exists('system/tmp/'.$this->captcha_session_code.'.txt') && !empty($captcha_pwd))
        {
          $this->captcha_check = $this->captcha->checkCaptcha($captcha_pwd, $this->captcha_session_code, 'system/tmp/');
          if ($this->captcha_check != 1)
          {
            $this->captcha->error_code = 'Falscher Captcha! Versuchen Sie es nochmal.';
          }
          else
          {
            @ unlink('tmp/'.$this->captcha_session_code.'.txt');
            $this->captcha->cleanTmp('system/tmp/');
          }
        }
        else
        {
          $this->captcha->error_code = 'Falscher Captcha! Versuchen Sie es nochmal.';
        }
      }
      $this->captcha_session_code = $this->captcha->newCaptcha('system/tmp/');
    }
  }


  /**
    * Funktion zum Ermitteln der vorherigen oder naechsten ID.
    *
    * @param    string $mode "last" oder "next"
    * @return   string
    * @access   private
    */
  private function getValidID($mode)
  {
    switch($mode)
    {
      // vorherige ID
      case 'last': $res = $this->sql->query("SELECT id, titel FROM ".DB_TABLE_news." WHERE id < '" . $this->news_id . "' AND published = '1' ORDER BY id desc LIMIT 1");
                break;
      // nchste ID
      case 'next': $res = $this->sql->query("SELECT id, titel FROM ".DB_TABLE_news." WHERE id > '" . $this->news_id . "' AND published = '1' ORDER BY id asc LIMIT 1");
                break;
      // fallback
      default: return false;
    }
    if($this->config['modrewrite'])
    {
      return intval($res[0]['id']).'-'.$res[0]['titel'];
    }
    else
    {
      return intval($res[0]['id']);
    }
  }

  /**
    * Ermitteln / Verarbeiten von Cookies fr das Kommentarformular.
    * $_COOKIE['EXP_COMMENTS_USERNAME'], $_COOKIE['EXP_COMMENTS_HOMEPAGE']
    *
    * @return   void
    * @access   private
    */
  private function checkCookies()
  {
    if($this->config['kommentare'])
    {
      if($_POST['remember'] == '1')
      {
        setcookie("EXP_COMMENTS_USERNAME", htmlentities($_POST['name']), time()+31536000);
        setcookie("EXP_COMMENTS_HOMEPAGE", htmlentities($_POST['homepage']), time()+31536000);

        $this->kommentar_name = htmlentities($_POST['name']);
        $this->kommentar_homepage = htmlentities($_POST['homepage']);
      }
      elseif($_GET['remember'] == '0')
      {
        if(isset($_COOKIE['EXP_COMMENTS_USERNAME']))
        {
          setcookie("EXP_COMMENTS_USERNAME", "", time()-3600);
        }
        if(isset($_COOKIE['EXP_COMMENTS_HOMEPAGE']))
        {
          setcookie("EXP_COMMENTS_HOMEPAGE", "", time()-3600);
        }

        $this->kommentar_name = 'Name';
        $this->kommentar_homepage = 'http://www.example.com';
      }
      else
      {
        if(isset($_COOKIE['EXP_COMMENTS_USERNAME']) && isset($_COOKIE['EXP_COMMENTS_HOMEPAGE']))
        {
          $this->kommentar_name = $this->sql->checkSql($_COOKIE['EXP_COMMENTS_USERNAME']);
          $this->kommentar_homepage = $this->sql->checkSql($_COOKIE['EXP_COMMENTS_HOMEPAGE']);
        }
        else
        {
          $this->kommentar_name = 'Name';
          $this->kommentar_homepage = 'http://www.example.com';
        }
      }
    }
  }


  /**
    * Destruktor
    *
    * @return   void
    * @access   public
    */
  public function __destruct() {}
}
?>