<?php
/**
  *  - system/classes/index.class.php -
  *
  *  @ Software    Expblog Weblog Software
  *  @ Version     0.6.2
  *  @ Lizenz      Freeware (siehe LIZENZ.txt im Hauptverzeichnis)
  *  @ Author      Volker S. Latainski
  *  @ Home        www.expblog.net / www.expblog.de
  *  @ Kontakt     kontakt@expblog.net
  *  @ Bugs        www.expblog.net/forum
  ***********************************************************************/
class INDEX extends OUTPUT
{
  /**
   * Array mit den Datensaetzen
   * @var array
   */
  public $data = array();
  /**
   * Parameter fuer die Sortierung (Usereingaben)
   * @var mixed
   */
  private $sortparam, $sortparam1, $sortparam2, $sortparam3;
  /**
   * Aktuelle Seite der Blaetterfunktion
   * @var int
   */
  public $page, $maxpage;
  /**
   * Maximal anzuzeigende Artikel pro Seite
   * @var int
   */
  private $pagelimit;
  /**
   * Flag ob der Pager angezeigt werden soll oder nicht
   * @var int
   */
  public $pagermode;
  /**
   * Anzahl aller Artikel ohne Limit
   * @var int
   */
  public $datacount = 0;
  /**
   * ---
   * @var string
   */
  public $pagerawurl = '';
  /**
   * ---
   * @var string
   */
  public $viewmode = '';

  /**
   * Konstruktor
   *
   * @param    array    $expconf Kopie der Blogkonfiguration
   * @return   void
   * @access   public
   */
  public function __construct($expconf)
  {
    parent::__construct($expconf); //Eigenschaften, Funktionen und Methoden der Elternklasse uebernehmen
    $this->connectSQL(); // Zur Datenbank verbinden
    $this->setViewMode();
    $this->setPage(); // Aktuelle Seite fuer die Blaetterfunktion ermitteln
    $this->setPageLimit(); // Maximale Anzahl der Artikel pro Seite festlegen
    $this->setData(); // Daten aus der Datenbank auslesen und in $this->data scheichern
    $this->setMaxPage();
  }
  
  /**
   * Aktuelle Seite fuer die Bltterfunktion(Pager) erkennen
   *
   * @return   void
   * @access   private
   */
  private function setMaxPage()
  {
    $this->maxpage = ceil(($this->datacount/$this->config['index_maxnews']));
  }
  
  /**
   * ---
   *
   * @return   void
   * @access   private
   */
  private function setViewMode()
  {
    // Archiv Modus
    if(isset($_GET['a']) && $_GET['a'] === 'archiv')
    {
      // Anzeigemodus tageweise
      if(isset($_GET['j']) && isset($_GET['m']) && isset($_GET['d']))
      {
        $this->viewmode = 'tag';
        $this->flag_position = 0;        
      }
      // Anzeigemodus monatsweise
      elseif(isset($_GET['j']) && isset($_GET['m']))
      {
        $this->viewmode = 'monat';
        $this->flag_position = 0;
      }
      // Anzeigemodus jahresweise
      elseif(isset($_GET['j']))
      {
        $this->viewmode = 'jahr';
        $this->flag_position = 0;
      }
      // default mode
      else
      {
        $this->viewmode = 'default';
        $this->flag_position = 2;
      }
    }
    // Kategorie Modus
    elseif((isset($_GET['a']) && $_GET['a'] === 'kategorie') || (isset($_GET['kategorie']) && !empty($_GET['kategorie'])))
    {
      $this->viewmode = 'kategorie';
      $this->flag_position = 3;
    }
    // autor mode
    elseif((isset($_GET['a']) && $_GET['a'] === 'autor') || (isset($_GET['autor']) && !empty($_GET['autor'])))
    {
      $this->viewmode = 'autor';
      $this->flag_position = 3;
    }
    // default mode
    else
    {
      $this->viewmode = 'default';
      $this->flag_position = 2;
    }
  }

  /**
   * Aktuelle Seite fuer die Bltterfunktion(Pager) erkennen
   *
   * @return   void
   * @access   private
   */
  private function setPage()
  {
    $flags = array('a','b','j','m','d','p');  
    if(is_numeric($_GET[$flags[$this->flag_position]]) || (isset($_GET['page']) && is_numeric($_GET['page'])))
    {
      if($_GET[$flags[$this->flag_position]] > 0)
      {      
        $this->page = intval(trim($_GET[$flags[$this->flag_position]]));
      }
      elseif($_GET['page'] > 0)
      {
        $this->page = intval(trim($_GET['page']));
      }
      else
      {
        $this->page = 1;
      }
    }
    else
    {
      $this->page = 1;
    }
  }

  /**
   * Aktuelle Seite fuer die Bltterfunktion(Pager) erkennen
   *
   * @return   void
   * @access   private
   */
  private function setPageLimit()
  {
    if($this->page > 1)
    {
      $start = ($this->page - 1) * $this->config['index_maxnews'];
      $this->pagelimit = 'LIMIT '.$start.', '.$this->config['index_maxnews'];
    }
    else
    {
      $this->pagelimit = 'LIMIT '.$this->config['index_maxnews'];
    }
  }

  /**
   * Funktion zum Auslesen der Daten, abhaengig vom Anzeigemodus
   *
   * @return   bool
   * @access   private
   */
  private function setData()
  {
    // Anzeigemodus tageweise
    if($this->viewmode == 'tag')
    {
      if(false !== ($data = $this->getDataFromDay(trim($_GET['j']), trim($_GET['m']), trim($_GET['d']))))
      {
        if($this->config['modrewrite'])
        {          
          $this->pagerawurl = WEBPATH.'archiv/'.$this->string2urlstring($_GET['j']).'/'.$this->string2urlstring($_GET['m']).'/'.$this->string2urlstring($_GET['d']).'/';
        }
        else
        {
          $this->pagerawurl = WEBPATH.'?autor='.$this->string2urlstring($_GET['autor']).'&amp;page=';
        }       
        $this->data = $data;
        return true;
      }
      return false;
    }    
    // Anzeigemodus monatsweise
    elseif($this->viewmode == 'monat')
    {
      if(false !== ($data = $this->getDataFromMonth(trim($_GET['j']), trim($_GET['m']))))
      {
        $this->data = $data;
        return true;
      }
      return false;
    }    
    // Anzeigemodus jahresweise
    elseif($this->viewmode == 'jahr')
    {
      if(false !== ($data = $this->getDataFromYear(trim($_GET['j']))))
      {
        $this->data = $data;
        return true;
      }
      return false;
    }
    // Kategorie Modus
    elseif($this->viewmode == 'kategorie')
    {
      if(false !== ($data = $this->getDataFromKategorie()))
      {
        if($this->config['modrewrite'])
        {          
          $this->pagerawurl = WEBPATH.'kategorie/'.$this->string2urlstring($_GET['j']).'/';
        }
        else
        {
          $this->pagerawurl = WEBPATH.'?kategorie='.$this->string2urlstring($_GET['kategorie']).'&amp;page=';
        }     
        $this->data = $data;
        return true;
      }
      return false;
    }
    // autor mode
    elseif($this->viewmode == 'autor')
    {
      if(false !== ($data = $this->getDataFromAutor()))
      {
        if($this->config['modrewrite'])
        {          
          $this->pagerawurl = WEBPATH.'autor/'.$this->string2urlstring($_GET['j']).'/';
        }
        else
        {
          $this->pagerawurl = WEBPATH.'?autor='.$this->string2urlstring($_GET['autor']).'&amp;page=';
        }        
        $this->data = $data;
        return true;
      }
      return false;
    }
    // default mode
    else
    {
      if(false !== ($data = $this->getDataFromDefault()))
      {
        if($this->config['modrewrite'])
        {
          $this->pagerawurl = WEBPATH.'page/';
        }
        else
        {
          $this->pagerawurl = WEBPATH.'?page=';
        }                
        $this->data = $data;
        return true;
      }
      return false;
    }
  }

  /**
   * Funktion zum Auslesen aller Artikel eines Tages
   *
   * @return   array|bool
   * @access   private
   */
  private function getDataFromDay($year, $month, $day)
  {
    // Datum checken und Ergebnis holen
    if(checkdate(intval($month), intval($day), intval($year)))
    {
      $this->sortparam = (int) $year;
      $this->sortparam2 = (int) $month;
      $this->sortparam3 = (int) $day;
      if($this->sortparam2 < 10)
      {
        $this->sortparam2 = '0'.$this->sortparam2;
      }
      if($this->sortparam3 < 10)
      {
        $this->sortparam3 = '0'.$this->sortparam3;
      }

      //Unterschiede der Queries, bedingt durch das verwendete DBMS
      if($this->config['dbms'] == 'mysql')
      {
        $term = "FROM_UNIXTIME(news.timestamp, '%Y-%m-%d')";
      }
      else
      {
        $term = "strftime('%Y-%m-%d', news.timestamp, 'unixepoch')";
      }
      
      $dataquery = "
        SELECT
          news.*,
          COUNT(kommentare.kommentar) AS kommentare_count,
          user.name AS username,
          kategorien.name AS kategorie
        FROM
          ".DB_TABLE_news." AS news
        LEFT JOIN
          ".DB_TABLE_kommentare." AS kommentare
        ON
          news.id = kommentare.news
        LEFT JOIN
          ".DB_TABLE_kategorien." AS kategorien
        ON
          news.kategorie = kategorien.id
        LEFT JOIN
          ".DB_TABLE_user." AS user
        ON
          user.id = news.userid
        WHERE
          news.published = 1
        AND
          ".$term." = '".$this->sortparam."-".$this->sortparam2."-".$this->sortparam3."'
        GROUP BY
          news.id
        ORDER BY
          news.timestamp
        DESC           
      ";
      $countquery = "
        SELECT
          COUNT(news.id) AS artikel_count
        FROM
          ".DB_TABLE_news." AS news        
        WHERE
          news.published = 1
        AND
          ".$term." = '".$this->sortparam."-".$this->sortparam2."-".$this->sortparam3."'         
      ";

      $res = $this->sql->query($dataquery);       
      if(!$res)
      {
        return false;
      }
      $datacount = $this->sql->query($countquery);
      $this->datacount = $datacount[0]['artikel_count'];
      return $res;    
    }
    else
    {
      return false;
    }
  }

  /**
   * Funktion zum Auslesen aller Artikel eines Monats
   *
   * @return   array|bool
   * @access   private
   */
  private function getDataFromMonth($year, $month)
  {
    // Datum checken und Ergebnis holen
    if(checkdate(1, intval($month), intval($year)))
    {
      $this->sortparam1 = (int) $year;
      $this->sortparam2 = (int) $month;

      //Unterschiede der Queries, bedingt durch das verwendete DBMS
      if($this->config['dbms'] == 'mysql')
      {
        $term1 = "YEAR(FROM_UNIXTIME(news.timestamp))";
        $term2 = "MONTH(FROM_UNIXTIME(news.timestamp))";
      }
      else
      {
        $term1 = "strftime('%Y', news.timestamp, 'unixepoch')";
        $term2 = "strftime('%m', news.timestamp, 'unixepoch')";
      }

      $dataquery = "
        SELECT
          news.*,
          COUNT(kommentare.kommentar) AS kommentare_count,
          user.name AS username,
          kategorien.name AS kategorie
        FROM
          ".DB_TABLE_news." AS news
        LEFT JOIN
          ".DB_TABLE_kommentare." AS kommentare
        ON
          news.id = kommentare.news
        LEFT JOIN
          ".DB_TABLE_kategorien." AS kategorien
        ON
          news.kategorie = kategorien.id
        LEFT JOIN
          ".DB_TABLE_user." AS user
        ON
          user.id = news.userid
        WHERE
          news.published = 1
        AND
          ".$term1." = '".$this->sortparam1."'
        AND
          ".$term2." = '".$this->sortparam2."'
        GROUP BY
          news.id
        ORDER BY
          news.timestamp
        DESC          
      ";
      $countquery = "
        SELECT
          COUNT(news.id) AS artikel_count
        FROM
          ".DB_TABLE_news." AS news        
        WHERE
          news.published = 1
        AND
          ".$term1." = '".$this->sortparam1."'
        AND
          ".$term2." = '".$this->sortparam2."'         
      ";

      $res = $this->sql->query($dataquery);       
      if(!$res)
      {
        return false;
      }
      $datacount = $this->sql->query($countquery);
      $this->datacount = $datacount[0]['artikel_count'];
      return $res;
    }
    else
    {
      return false;
    }
  }

  /**
   * Funktion zum Auslesen aller Artikel eines Jahres
   *
   * @return   array|bool
   * @access   private
   */
  private function getDataFromYear($year)
  {
    // Datum checken und Ergebnis holen
    if(checkdate(1, 1, intval($year)))
    {
      $this->sortparam = (int) $year;

      // Unterschiede der Queries, bedingt durch das verwendete DBMS
      if($this->config['dbms'] == 'mysql')
      {
        $term = "YEAR(FROM_UNIXTIME(news.timestamp))";
      }
      else
      {
        $term = "strftime('%Y', news.timestamp, 'unixepoch')";
      }
      
      $dataquery = "
        SELECT
          news.*,
          COUNT(kommentare.kommentar) AS kommentare_count,
          user.name AS username,
          kategorien.name AS kategorie
        FROM
          ".DB_TABLE_news." AS news
        LEFT JOIN
          ".DB_TABLE_kommentare." AS kommentare
        ON
          news.id = kommentare.news
        LEFT JOIN
          ".DB_TABLE_kategorien." AS kategorien
        ON
          news.kategorie = kategorien.id
        LEFT JOIN
          ".DB_TABLE_user." AS user
        ON
          user.id = news.userid
        WHERE
          news.published = 1
        AND
          ".$term." = '".$this->sortparam."'
        GROUP BY
          news.id
        ORDER BY
          news.timestamp
        DESC
      ";
      $countquery = "
        SELECT
          COUNT(news.id) AS artikel_count
        FROM
          ".DB_TABLE_news." AS news        
        WHERE
          news.published = 1
        AND
          ".$term." = '".$this->sortparam."'
      ";
            
      $res = $this->sql->query($dataquery);       
      if(!$res)
      {
        return false;
      }
      $datacount = $this->sql->query($countquery);
      $this->datacount = $datacount[0]['artikel_count'];
      return $res;
      }
    else
    {
      return false;
    }
  }

  /**
   * Funktion zum Auslesen der Artikel einer bestimmten Kategorie
   *
   * @return   array|bool
   * @access   private
   */
  private function getDataFromKategorie()
  {
    if(!empty($_GET['j']))
    {
      $this->sortparam = $this->sql->checkSQL(urldecode(trim($_GET['j'])));
    }
    elseif(!empty($_GET['kategorie']))
    {
      $this->sortparam = $this->sql->checkSQL(urldecode(trim($_GET['kategorie'])));
    }

    if(!empty($this->sortparam))
    {
      $dataquery = "
        SELECT
          news.*,
          COUNT(kommentare.kommentar) AS kommentare_count,
          user.name AS username,
          kategorien.name AS kategorie
        FROM
          ".DB_TABLE_news." AS news
        LEFT JOIN
          ".DB_TABLE_kommentare." AS kommentare
        ON
          news.id = kommentare.news
        LEFT JOIN
          ".DB_TABLE_user." AS user
        ON
          user.id = news.userid
        LEFT JOIN
          ".DB_TABLE_kategorien." AS kategorien
        ON
          news.kategorie = kategorien.id
        WHERE
          kategorien.name = '".$this->sortparam."'
        AND
          news.published = 1
        GROUP BY
          news.id
        ORDER BY
          news.timestamp
        DESC
          ".$this->pagelimit."
      ";
      $countquery = "
        SELECT
          COUNT(news.id) AS artikel_count
        FROM
          ".DB_TABLE_news." AS news
        LEFT JOIN
          ".DB_TABLE_kategorien." AS kategorien
        ON
          news.kategorie = kategorien.id
        WHERE
          kategorien.name = '".$this->sortparam."'
        AND
          news.published = 1
      ";
      
      $res = $this->sql->query($dataquery);  
    }       
    if(!$res)
    {
      return false;
    }
    $datacount = $this->sql->query($countquery);
    $this->datacount = $datacount[0]['artikel_count'];
    return $res;
  }

  /**
   * ----
   *
   * @return   array|bool
   * @access   private
   */
  private function getDataFromAutor()
  {
    if(!empty($_GET['j']))
    {
      $this->sortparam = $this->sql->checkSQL($this->stringFromUrlstring($_GET['j']));
    }
    elseif(!empty($_GET['autor']))
    {
      $this->sortparam = $this->sql->checkSQL($this->stringFromUrlstring($_GET['autor']));
    }

    if(!empty($this->sortparam))
    {
      $dataquery = "
        SELECT
          news.*,
          COUNT(kommentare.kommentar) AS kommentare_count,
          user.name AS username,
          kategorien.name AS kategorie
        FROM
          ".DB_TABLE_news." AS news
        LEFT JOIN
          ".DB_TABLE_kommentare." AS kommentare
        ON
          news.id = kommentare.news
        LEFT JOIN
          ".DB_TABLE_kategorien." AS kategorien
        ON
          news.kategorie = kategorien.id
        LEFT JOIN
          ".DB_TABLE_user." AS user
        ON
          user.id = news.userid
        WHERE
          news.userid = user.id
        AND
          user.name = '".$this->sortparam."'
        AND
          news.published = 1
        GROUP BY
          news.id
        ORDER BY
          news.timestamp
        DESC
          ".$this->pagelimit."
      ";
      
      $countquery = "
        SELECT
          COUNT(news.id) AS artikel_count
        FROM
          ".DB_TABLE_news." AS news
        LEFT JOIN
          ".DB_TABLE_user." AS user
        ON
          user.id = news.userid
        WHERE
          news.userid = user.id
        AND
          user.name = '".$this->sortparam."'
        AND
          news.published = 1
      ";
      
      $res = $this->sql->query($dataquery);   
    }      
     
    if(!$res)
    {
      return false;
    }
    $datacount = $this->sql->query($countquery);
    $this->datacount = $datacount[0]['artikel_count'];
    return $res;
  }

  /**
   * Funktion Auslesen aller Artikel (Standardausgabe)
   *
   * @return   array|bool
   * @access   private
   */
  private function getDataFromDefault()
  {
    $dataquery = "
      SELECT
        news.*,
        COUNT(kommentare.kommentar) AS kommentare_count,
        user.name AS username,
        kategorien.name AS kategorie
      FROM
        ".DB_TABLE_news." AS news
      LEFT JOIN
        ".DB_TABLE_kommentare." AS kommentare
      ON
        news.id = kommentare.news
      LEFT JOIN
        ".DB_TABLE_kategorien." AS kategorien
      ON
        news.kategorie = kategorien.id
      LEFT JOIN
        ".DB_TABLE_user." AS user
      ON
        user.id = news.userid
      WHERE
        news.published = 1
      AND
        news.startsite = 1
      GROUP BY
        news.id
      ORDER BY
        news.timestamp
      DESC
        ".$this->pagelimit."
    ";
    
    $countquery = "
      SELECT
        COUNT(news.id) AS artikel_count
      FROM
        ".DB_TABLE_news." AS news
      WHERE
        news.published = 1
      AND
        news.startsite = 1
    ";
    
    $res = $this->sql->query($dataquery);  
    if(!$res)
    {
      return false;
    }
    $datacount = $this->sql->query($countquery);
    $this->datacount = $datacount[0]['artikel_count'];
    return $res;
  }

  /**
    * Destruktor
    *
    * @return   void
    * @access   public
    */
  public function __destruct() {}
}
?>