<?php
/**
 * Expblog Frontend Basisklasse
 *
 * Diese Klasse ist zum Initialisieren der Ausgabe des Expblogs notwendig.
 * @package     Expblog
 * @category    Backend
 * @author      Volker S. Latainski
 * @copyright   Copyright (c) 2008, Volker S. Latainski
 * @version     0.6.3
 * @license     Freeware (siehe LIZENZ.txt im Hauptverzeichnis)
 * @link        http://www.expblog.net
 * @link        http://www.expblog.de
 * @filesource
 */
class OUTPUT
{
  /**
   * Kopie des expconf Array aus system/expconfig.php
   * @var array
   */
  public  $config;
  /**
   * Kopie der Konstante WEBPATH(URL zum Blogverzeichnis)
   * @var string
   */
  public  $webpath;
  /**
   * URL zum RSS Feed (i.d.R. rss.xml im Blogverzeichnis)
   * @var string
   */
  public  $rsspath;
  /**
   * Zeichen die in URLs nichts zu suchen haben, wird fr Mod_Rewrite bentigt!
   * @var string
   * @todo Ergaenzen!
   */
  public  $badinurl = array(" ", ".", "/", "%");
  /**
   * Startzeitpunkt der Script-Laufzeitmessung
   * @var float
   */
  private $startstamp;
  /**
   * Datenbankobjekt. Instanz der Klasse MYSQL.
   * @var object
   */
  public  $sql;
  /**
   * Array mit allen verfgbaren Sprachdateien des aktuell ausgewaehlten Templates.
   * @var array
   */
  public  $languagefiles;
  /**
   * SQLite Datenbank Datei
   * @var float
   */
  public  $sqlitefile = 'system/expblog.sqlite.db';


  /**
   * Konstruktor
   *
   * @param    array $expconf Kopie der Blogkonfiguration(system/expconf.php)
   * @param    bool $init Schalter: Blog initialisieren (true) oder "normales" Output Objekt erzeugen (false)
   * @return   void
   * @access   public
   */
  public function __construct($expconf, $init=false)
  {
    $this->config = $expconf;
    if($init)
    {
      $this->initExpblog();
    }
  }

  /**
   * Funktion zum Initialisieren des Expblog. Bitte nicht benutzen beim
   * Aufruf des Output Konstruktors in vererbten Klassen (index,details,ect.)!
   *
   * @return   void
   * @access   private
   */
  private function initExpblog()
  {
    $this->checkInstalldir();
    $this->checkWartungsmodus();
    $this->setEnvironment();
    $this->connectSQL();
    $this->check4News();
    $this->startstamp = $this->getMicroTime();
    $this->webpath = WEBPATH;
    $this->rsspath = WEBPATH.'rss.xml';
  }

  /**
   * Funktion zum Aufbauen der MYSQL Datenbankverbindung. Es werden die Zugangsdaten
   * aus der Datei system/mysqlconf.php verwendet.
   *
   * @return   void
   * @access   public
   */
  public function connectSQL()
  {
    if($this->config['dbms'] === 'mysql')
    {
      $this->sql = new MYSQL;
      $this->sql->connect(DATABASE, HOST, USERNAME, PASSWORD);
    }
    elseif($this->config['dbms'] === 'sqlite')
    {
      $this->sql = new SQLITE;
      $this->sql->connect(ABSPATH.'system/'.$this->sqlitefile, false, false, false);
    }
  }

  /**
   * Zeitzone auf Deutschland setzen und Seperator Workaround zum
   * Ueberschreiben der PHP Konfiguration. (Funktioniert nicht mit
   * jeder Serverkonfiguration)
   *
   * @return   void
   * @access   private
   */
  private function setEnvironment()
  {
    @ setlocale(LC_ALL, 'de_DE@euro', 'de_DE.iso-8859-1', 'de_DE', 'de', 'ge', 'deu_deu', 'German_Germany');
    @ ini_set("arg_separator.output", "&amp;");
  }

  /**
   * Ueberpruefung ob der Wartungsmodus eingeschaltet ist.
   *
   * @return   void
   * @access   private
   */
  private function checkWartungsmodus()
  {
    if($this->config['wartungsmodus'] && !$this->config['offlinesite'])
    {
      header("location: ".WEBPATH."offline.php");
      exit;
    }
  }

  /**
   * Existiert das Installations-Verzeichnis -> dann Weiterleitung
   *
   * @return   void
   * @access   public
   */
  public function checkInstalldir()
  {
    if(file_exists("install/"))
    {
      header("location: install/index.php");
      exit;
    }
  }

  /**
   * Funktion zum Ermitten aller verfgbaren Sprachdateien.
   *
   * @return   array
   * @access   public
   * @todo Funktion in das ACP integrieren.
   */
  public function getLanguageFiles()
  {
    $languagefiles = array();
    if(!$handle = @ opendir($this->config['style_dir'].$this->config['style'].'/'.'language/'))
    {
      return false;
    }
    else
    {
      while(false !== ($file = readdir($handle)))
      {
        if($file != "." && $file != "..")
        {
          @ list($filename, $fileprefix, $filetype) = explode(".", $file);
          if($fileprefix == "lang" && $filetype == "php")
          {
            array_push($languagefiles, $filename);
          }
        }
      }
      closedir($handle);
    }

    return $languagefiles;
  }

  /**
   * Funktion zum berprfen ob Posts in der Warteschlange sind und
   * freigegeben werden knnen.
   *
   * @return   bool
   * @access   private
   */
  public function check4News()
  {
    $res = $this->sql->query("SELECT count(id) FROM ".DB_TABLE_warteschlange);

    if($res[0][0] > 0)
    {
      $time = time();
      $result = $this->sql->query("SELECT * FROM ".DB_TABLE_warteschlange." WHERE timestamp <= {$time} ORDER BY timestamp");
      foreach($result as $news)
      {
        $add = $this->sql->query("
            INSERT INTO
              ".DB_TABLE_news."
            (
              timestamp,
              userid,
              kategorie,
              titel,
              teaser,
              nachricht,
              ip,
              startsite,
              published
            )
            VALUES
            (
              {$news[1]},
              {$news[2]},
              {$news[3]},
              '{$news[4]}',
              '{$news[5]}',
              '{$news[6]}',
              '{$news[7]}',
              {$news[8]},
              1
            )
        ");
        $del = $this->sql->query("DELETE FROM ".DB_TABLE_warteschlange." WHERE id={$news[0]}");
      }
      $this->makeRSSFeed();
      return true;
    }

    return false;
  }

  /**
   * Funktion zum schreiben des RSS Feed's.
   *
   * @return   bool
   * @access   private
   */
  private function makeRSSFeed()
  {
    if($this->config['rssstatus'])
    {
      $rss = new RSS($this);
      $makefeed = $rss->makeFeed();
      $rss = NULL;
      unset($rss);
      return true;
    }
    return false;
  }


  /**
   * Wichtige Funktion fr den RSS Feed!
   * Umwandeln unerwnschter Zeichen(HTML) in ihre numerischen Entitten.
   *
   * @param    string $string String der umgewandelt werden soll
   * @return   string
   * @access   public
   */
  public function htmlnumericentities($string)
  {
    return preg_replace('/[^!-%\x27-;=?-~ ]/e', '"&#".ord("$0").chr(59)', $string);
  }


  /**
   * Plugin History aus der Datei plugins/history.txt auslesen. Dieses
   * legt die aktiven Plugins und ihre Reihenfolge fest.
   *
   * @return   array|bool Rckgabe der aktiven Plugins. Array oder false.
   * @access   public
   */
  private function getPluginhistory()
  {
    $pluginhistory = array();
    if(false !== ($ph = fopen("plugins/history.txt", 'r')))
    {
      while(!feof($ph))
      {
        $plugin = fgets($ph, 4096);
        array_push($pluginhistory, rtrim($plugin));
      }
      fclose($ph);
      return $pluginhistory;
    }
    return false;
  }

  /**
   * Methode zum Includieren und Ausgeben der Plugins
   *
   * @return   void
   * @access   public
   */
  public function writePlugins($output, $lang)
  {
    $pluginhistory = $this->getPluginhistory();

    // Signatur der Plugin Funktionen, bergeben als Array
    $signatur = array(
    "output" => $output,
    "lang" => $lang,
    "_POST" => $_POST,
    "_GET" => $_GET
    );

    // Plugins includieren
    foreach($pluginhistory as $plugin)
    {
      $pluginpfad = 'plugins/'.$plugin.'.php';
      if(file_exists($pluginpfad))
      {
        include_once $pluginpfad;
      }
      @call_user_func_array($plugin, $signatur);
    }
  }

  /**
   * Video Parser
   *
   * @param    string $text
   * @return   string
   * @access   public
   */
  public function videoParser($text)
  {
    $ersatz = array(
    'youtube'   => '<object width="'.$this->config['videoabmessung_b'].'" height="'.$this->config['videoabmessung_h'].'"><param name="movie" value="http://www.youtube.com/v/videocode"></param><param name="wmode" value="transparent"></param><embed src="http://www.youtube.com/v/videocode" type="application/x-shockwave-flash" wmode="transparent" style="width:'.$this->config['videoabmessung_b'].'px; height:'.$this->config['videoabmessung_h'].'px;"></embed></object>',
    'google'    => '<object width="'.$this->config['videoabmessung_b'].'" height="'.$this->config['videoabmessung_h'].'"><embed style="width:'.$this->config['videoabmessung_b'].'px; height:'.$this->config['videoabmessung_h'].'px;" id="VideoPlayback" type="application/x-shockwave-flash" src="http://video.google.com/googleplayer.swf?docId=videocode&hl=de" flashvars=""></embed></object>',
    'myvideo'   => '<object width="'.$this->config['videoabmessung_b'].'" height="'.$this->config['videoabmessung_h'].'"><param name="movie" value="http://www.myvideo.de/movie/videocode"></param><embed src="http://www.myvideo.de/movie/videocode" width="'.$this->config['videoabmessung_b'].'" height="'.$this->config['videoabmessung_h'].'" type="application/x-shockwave-flash"></embed></object>',
    'clipfish'  => '<object width="'.$this->config['videoabmessung_b'].'" height="'.$this->config['videoabmessung_h'].'"><param name="movie" value="http://www.clipfish.de/videoplayer.swf?as=0&videoid=videocode&r=1" /><param name="wmode" value="transparent"><param name="quality" value="high" /><param name="bgcolor" value="#999999" /><embed src="http://www.clipfish.de/videoplayer.swf?as=0&videoid=videocode&r=1" quality="high" bgcolor="#999999" width="'.$this->config['videoabmessung_b'].'" height="'.$this->config['videoabmessung_h'].'" name="player" align="middle" allowScriptAccess="sameDomain" type="application/x-shockwave-flash" pluginspage="http://www.macromedia.com/go/getflashplayer"></embed></object>'
    );

    $treffer = array();
    preg_match_all('/\[video(.*)\]/isU', $text, $treffer, PREG_OFFSET_CAPTURE);

    foreach($treffer[0] as $treffer)
    {
      list($tag, $url, $videocode) = explode('=', $treffer[0]);

      if(strpos($url, 'google'))
      {
        $base = 'google';
        $replacement = str_replace('videocode', substr(trim($videocode), 0, -1), $ersatz[$base]);
      }
      elseif(strpos($url, 'youtube'))
      {
        $base = 'youtube';
        $replacement = str_replace('videocode', substr(trim($videocode), 0, -1), $ersatz[$base]);
      }
      elseif(strpos($url, 'myvideo'))
      {
        $base = 'myvideo';
        $split = explode('/', $url);
        $replacement = str_replace('videocode', trim($split[4]), $ersatz[$base]);
      }
      elseif(strpos($url, 'clipfish'))
      {
        $base = 'clipfish';
        $replacement = str_replace('videocode', substr(trim($videocode), 0, -1), $ersatz[$base]);
      }

      $text = str_replace($treffer[0], $replacement, $text);
    }

    return $text;
  }

  /**
   * Funktion zum ermitteln des aktuellen Zeitstempels in Microsekunden
   *
   * @return   float Aktueller Zeitstempel in Microsekunden
   * @access   private
   */
  private function getMicroTime()
  {
    list($sec, $nanosec) = explode(" ", microtime());
    return ((float)$sec + (float)$nanosec);  //Typenumwandlung
  }

  /**
   * Funktion zum ermitteln der Scriptlaufzeit in Sekunden.
   *
   * @param    int $digits Anzahl der Stellen hinter dem Komma
   * @return   float
   * @access   public
   */
  public function getRuntime($digits)
  {
    $runtime = $this->getMicroTime() - $this->startstamp;
    $runtime = round($runtime, $digits);
    return $runtime;
  }


  /**
   * Reloadsperre fr News bzw. Kommentare. (ACP & Frontend)
   *
   * @param    string $checkme Der zu berprfende String
   * @param    int $mode Modus der Sperre 0=News, 1=Kommentare, 2=Warteschlange
   * @return   bool
   * @access   protected
   */
  protected function checkLast($checkme, $mode)
  {
    // Modus f. Sperre ermitteln;
    switch($mode)
    {
      case 0:  $res = $this->sql->query("SELECT titel FROM ".DB_TABLE_new." ORDER BY id DESC LIMIT 5");
                 break;
      case 1:  $res = $this->sql->query("SELECT kommentar FROM ".DB_TABLE_kommentare." ORDER BY id DESC LIMIT 5");
                 break;
      case 2:  $res = $this->sql->query("SELECT titel FROM ".DB_TABLE_warteschlange." ORDER BY id DESC LIMIT 5");
                 break;
      default:   return false; // fallback
    }
    // Vergleich von $checkme mit den letzten Eintrgen
    foreach($res as $row => $value)
    {
      if(trim($checkme) === trim($res[0][0]))
      {
        return true;
      }
    }
    return false;
  }


  /**
   * Eine Funktion zum validieren des richtigen Formats einer URL.
   *
   * @param    string $url zu pruefende URL
   * @return   bool
   * @access   public
   */
  public function isURL($url)
  {
    if(!preg_match('#^http\\:\\/\\/[a-z0-9\-]+\.([a-z0-9\-]+\.)?[a-z]+#i', $url))
    {
      return false;
    }
    else
    {
      return true;
    }
  }

  /**
   * Eine Funktion zum Ermitteln des Benutzerlevels zu einem Usernamen.
   *
   * @param    string $username Benutzername
   * @return   int
   * @access   public
   */
  public function getUserlevel($username)
  {
    $result = $output->sql->query("SELECT level FROM ".DB_TABLE_user." WHERE name='".$this->sql->checkSql($username)."'");
    return $userlevel[0][0];
  }

  /**
   * Funktion zum Vorbereiten von Strings auf die Verwendung in URLs
   *
   * @param    string $string
   * @return   string
   * @access   public
   * @todo     Umlaute rausfiltern; badinurls erweitern
   */
  public function string2urlstring($string)
  {
    return urlencode(str_replace(".", ",", trim($string)));
  }

  /**
   * Funktion zum Dekodieren eines Strings aus einer URL.
   *
   * @param    string $string
   * @return   string
   * @access   public
   * @todo     Umlaute rausfiltern; badinurls erweitern
   */
  public function stringFromUrlstring($string)
  {
    return str_replace(",", ".", urldecode($string));
  }

  /**
   * Destruktor
   *
   * @return   void
   * @access   public
   */
  public function __destruct() {}

}
?>