<?php
/**
 * Diese RPC-Ping Klasse stellt alle Funktionalitt fr RPC-Pings bereit.
 *
 * @package     Expblog
 * @category    Backend
 * @author      Volker S. Latainski
 * @copyright   Copyright (c) 2008, Volker S. Latainski
 * @version     0.6
 * @license     Freeware (siehe LIZENZ.txt im Hauptverzeichnis)
 * @link        http://www.expblog.net
 * @link        http://www.expblog.de
 * @filesource
 */
class RPCPING
{
  /**
   * Kopie von DB_TABLE_rpcping
   * @var string
   */
  private $expblog_rpcping;
  /**
   * Useragent(eine Art Signatur) des Blogs
   * @var array
   */
  private $useragent;
  /**
   * Array mit den Diensten die angepingt werden sollen
   * @var array
   */
  private $pinglist = array();
  /**
   * Timeout der Anfragen/Pings in Sekunden
   * @var array
   */
  private $ping_timeout = 30;
  /**
   * Referenz auf das $output Objekt des Blogs
   * @var reference|object
   */
  private $output;
  /**
   * Benutzerlevel des Autors / Kopie $_SESSION['userlevel']
   * @var int
   */
  private $userlevel;

  /**
   * Konstruktor
   *
   * @param    reference|object $output Referenz auf das $output Objekt des Blogs
   * @return   void
   * @access   public
   */
  public function __construct($output)
  {
    $this->output = $output;
    $this->useragent = 'expblog-'.VERSION;
    $this->expblog_rpcping = DB_TABLE_rpcping;
    $this->userlevel  = $_SESSION['userlevel'];
    $this->readPinglist();
  }

  /**
   * Funktion zum Anpingen aller Dienste der Pingliste
   *
   * @return   array
   * @access   public
   */
  public function ping()
  {
    $xmlreq = '<?xml version="1.0"?>
            <methodCall>
              <methodName>weblogUpdates.ping</methodName>
              <params>
                <param>
                  <value>'.$this->output->config['blogtitel'].'</value>
                </param>
                <param>
                  <value>'.WEBPATH.'</value>
                </param>
              </params>
            </methodCall>
    ';

    $pingresponse = array();
    foreach($this->pinglist as $service)
    {
      $header  = "POST ".$service['path']." HTTP/1.0\r\n";
      $header .= "User-Agent: ".$this->useragent."\r\n";
      $header .= "Host: ".$service['host']."\r\n";
      $header .= "Content-Type: text/xml\r\n";
      $header .= "Content-length: ".strlen($xmlreq)."\r\n\r\n";
      $header .= $xmlreq."\r\n";

      $handle = fsockopen($service['host'], 80, $errno, $errstr, $this->ping_timeout);
      if(!$handle)
      {
        return $errno.':'.$errstr;
      }
      else
      {
        fputs($handle, $header);
        $response = array();
        array_push($response, $service['host']);
        while(!feof($handle))
        {
          $respo = fgets($handle, 128);
          array_push($response, $respo);
        }
        fclose($handle);
        array_push($pingresponse, $response);
      }
    }
    return $pingresponse;
  }

  /**
   * Funktion zum Einlesen aller Dienste in die Pingliste
   *
   * @return   void
   * @access   private
   */
  private function readPinglist()
  {
    $res = $this->output->sql->query("
        SELECT
          id, url, titel
        FROM
          {$this->expblog_rpcping}
        WHERE
          {$this->userlevel} > 2
    ");
    if($res)
    {
      $parsed_url = parse_url(trim($res[0]['url']));
      $this->pinglist[$parsed_url['host']] = $parsed_url;
    }
  }

  /**
   * Funktion zum Hinzufgen eines RPC-Dienstes in die Datenbank
   *
   * @param    string $url   URL des Dienstes
   * @param    string $titel Optionaler Titel des Dienstes
   * @return   bool
   * @access   public
   */
  public function addHost($url, $titel)
  {
    if(!empty($url))
    {
      $res = $this->output->sql->query("
          INSERT INTO
            {$this->expblog_rpcping}
          (
            url,
            titel
          )
          VALUES
          (
            '{$url}',
            '{$titel}'
          )
      ");
      if(is_array($res) || $res === true)
      {
        return true;
      }
      return false;
    }
    return false;
  }

  /**
   * Funktion zum Lschen eines RPC-Dienstes aus der Datenbank
   *
   * @param    int $id ID des zu lschenden Dienstes
   * @return   bool
   * @access   public
   */
  public function deleteHost($id)
  {
    $res = $this->output->sql->query("
        DELETE FROM
          {$this->expblog_rpcping}
        WHERE
          id = {$id}
        AND
          {$this->userlevel} > 2
    ");
    if(is_array($res) || $res === true)
    {
      return true;
    }
    return false;
  }

  /**
   * Funktion zum Auslesen aller RPC-Dienstes aus der Datenbank
   *
   * @return   array|bool
   * @access   private
   */
  public function getAll()
  {
    $res = $this->output->sql->query("
        SELECT
          id,
          url,
          titel
        FROM
          {$this->expblog_rpcping}
        WHERE
          {$this->userlevel} > 2
    ");
    if(is_array($res) || $res === true)
    {
      return $res;
    }
    return false;
  }

  /**
   * Destruktor
   *
   * @return   void
   * @access   public
   */
  public function __destruct() {}
}
?>